import json
import time, sys, os
from seleniumwire import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as ec
from webdriver_manager.chrome import ChromeDriverManager
from getPlants import saveJson
from pprint import pprint as pp


def raiseException(err):
    print(err)
    exc_type, exc_obj, exc_tb = sys.exc_info()
    file_name = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
    print(exc_type, file_name, exc_tb.tb_lineno)


def download(name, driver):
    products = []
    for request in driver.requests:
        products.append(json.loads(request.body.decode('utf-8')))
    with open(f'request_{name}.json', 'w') as f:
        json.dump(products, f, indent=4)
    saveJson(name)
    wait = WebDriverWait(driver, 60)
    wait.until(ec.element_to_be_clickable((By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div')))
    driver.find_element(By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div').click()
    print('Clicking Export Price List')
    wait.until(
        ec.element_to_be_clickable((By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div/div/a[3]')))
    driver.find_element(By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div/div/a[3]').click()


def waitForOk(driver):
    wait = WebDriverWait(driver, 60)

    ok_btn_xpath = '//html/body/ngb-modal-window/div/div/app-share-price-lists/div[3]/button[2]'
    driver.find_element(By.XPATH, ok_btn_xpath).click()
    print('Clicked OK...', end='')
    try:
        print('Waiting for download to finish.')
        progress = '//html/body/ngb-modal-window/div/div/app-share-price-lists/div[3]/button[2]/span[2]'
        wait.until(ec.presence_of_element_located((By.XPATH, progress)))

        time.sleep(1)
        runn = True
        while runn:
            try:
                driver.find_element(By.CSS_SELECTOR, '.ladda-progress')
                # while driver.find_element(By.XPATH, progress).get_attribute('innerHTML') != '':
                print('waiting 5s..')
                time.sleep(5)
            except:
                runn = False

    except Exception as e:
        raiseException(e)


def singleDownload(driver, download_type=1):
    wait = WebDriverWait(driver, 60)
    wait.until(ec.element_to_be_clickable((By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div')))
    driver.find_element(By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div').click()
    print('Clicking Export Price List')
    wait.until(ec.element_to_be_clickable((By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div/div/a[3]')))
    driver.find_element(By.XPATH, '//*[@id="navbarNavDropdown"]/div/div[1]/div[2]/div/div/a[3]').click()

    flower_xpath = '//html/body/ngb-modal-window/div/div/app-share-price-lists/div[2]/div/ul/li[2]/span'
    plants_xpath = '//html/body/ngb-modal-window/div/div/app-share-price-lists/div[2]/div/ul/li[3]/span'
    decoration_xpath = '//html/body/ngb-modal-window/div/div/app-share-price-lists/div[2]/div/ul/li[4]/span'
    # if download_type == 1:
    print("Saving Auth: ", driver.last_request.headers.get('authorization'))
    with open('auth.txt', 'w', encoding='utf-8') as f:
        f.write(driver.last_request.headers.get('authorization'))
    print('Downloading Flower')
    del driver.requests
    wait.until(ec.element_to_be_clickable((By.XPATH, flower_xpath)))
    driver.find_element(By.XPATH, flower_xpath).click()
    waitForOk(driver)
    download('flowers', driver)

    # elif download_type == 2:
    print('Downloading Plants HAR')
    wait.until(ec.element_to_be_clickable((By.XPATH, plants_xpath)))
    del driver.requests
    driver.find_element(By.XPATH, plants_xpath).click()
    waitForOk(driver)
    download('plants', driver)

    # elif download_type == 3:
    print('Downloading Decorations HAR')
    wait.until(ec.element_to_be_clickable((By.XPATH, decoration_xpath)))
    del driver.requests
    driver.find_element(By.XPATH, decoration_xpath).click()
    waitForOk(driver)
    download('decorations', driver)

    # if download_type == 1:
    # elif download_type == 2:
    #
    # elif download_type == 3:
    #
    print('downloaded!')
    driver.quit()


def driver_prep():
    options = webdriver.ChromeOptions()
    options.add_argument('--ignore-ssl-errors=yes')
    options.add_argument('--ignore-certificate-errors')
    options.add_argument('--headless')
    options.add_argument('--no-gpu')
    user_agent = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
    options.add_argument(f'--user-agent={user_agent}')
    prefs = {
        "download_restrictions": 3,
    }
    options.add_experimental_option(
        "prefs", prefs
    )
    enable_har = {
        'enable_har': True  # Capture HAR data, retrieve with driver.har
    }

    driver = webdriver.Chrome(ChromeDriverManager().install(), chrome_options=options, seleniumwire_options=enable_har)
    driver.scopes = [
        '.*/servoy-service/rest_ws/axws/rListRows/-1/false/2.*'
    ]
    driver.set_window_size(1366, 768)
    print('Logging in.')
    driver.get('https://www.floralwebshop.com/')

    wait = WebDriverWait(driver, 120)
    wait.until(ec.presence_of_element_located((By.ID, 'username')))
    wait.until(ec.presence_of_element_located((By.ID, 'password')))
    wait.until(ec.element_to_be_clickable((By.ID, 'login')))
    driver.find_element(By.ID, 'username').send_keys('010412')
    driver.find_element(By.ID, 'password').send_keys('Juzar1')
    driver.find_element(By.ID, 'login').click()
    print('clicked sign in')
    try:
        wait.until(ec.element_to_be_clickable((By.ID, 'ok-btn')))
        driver.find_element(By.ID, 'ok-btn').click()
    except:
        print('No Ok')
        exit(0)
    print('Successfully Logged in.')

    loading_state_xpath = '//html/body/app-root/app-axe-layout/app-shop/div[1]/div[1]'
    wait.until(ec.presence_of_element_located((By.XPATH, loading_state_xpath)))
    print('Waiting for content to load!')
    while driver.find_element(By.XPATH, loading_state_xpath).text.strip() == 'Loading...':
        time.sleep(5)
    print('Content Loaded. Waiting 5s..')
    time.sleep(5)
    return driver


def main():
    singleDownload(driver_prep())
    # singleDownload(driver_prep(), 2)
    # singleDownload(driver_prep(), 3)


if __name__ == '__main__':
    main()
